#include <allegro.h>
#include <winalleg.h>
#include "Collide.h"
#include "Vectors.h"
#include "dll_shared.h"

collision_map_p CreateCollisionMap()
{
	collision_map_p pMap = new collision_map_t;

	return pMap;
}

void DeleteCollisionMap(collision_map_p& pMap)
{
	pMap->svColliders.clear();
	pMap->svPoints.clear();

	delete pMap;

	pMap = NULL;
}

void InsertCollider(collision_map_p pMap, collider_p pCol)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_NONE)
		{
			*pPos = *pCol;
			return;
		}
	}

	pMap->svColliders.push_back(*pCol);
}

int FindColliderPoint(collision_map_p pMap, int dwX, int dwY)
{
	for(col_point_list_t::iterator pPos=pMap->svPoints.begin(); pPos!=pMap->svPoints.end(); ++pPos)
	{
		if(pPos->dwvPos[0] == dwX  &&  pPos->dwvPos[1] == dwY)
			return pPos-pMap->svPoints.begin();
	}

	col_point_t sPnt;
	sPnt.dwvPos[0] = dwX;
	sPnt.dwvPos[1] = dwY;
	
	pMap->svPoints.push_back(sPnt);

	return pMap->svPoints.size()-1;
}

int FindBlocker(collision_map_p pMap, int dwP1, int dwP2)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_BLOCKER  &&  pPos->dwvInfo[0] == dwP1  &&  pPos->dwvInfo[1] == dwP2)
			return pPos-pMap->svColliders.begin();
	}

	return -1;
}

int FindLeftGrab(collision_map_p pMap, int dwP1)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_LEFT_GRAB  &&  pPos->dwvInfo[0] == dwP1)
			return pPos-pMap->svColliders.begin();
	}

	return -1;
}

int FindRightGrab(collision_map_p pMap, int dwP1)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_RIGHT_GRAB  &&  pPos->dwvInfo[0] == dwP1)
			return pPos-pMap->svColliders.begin();
	}

	return -1;
}

int FindPole(collision_map_p pMap, int dwP1, int dwP2)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_POLE  &&  (pPos->dwvInfo[0] == dwP1  &&  pPos->dwvInfo[1] == dwP2  ||  pPos->dwvInfo[0] == dwP2  &&  pPos->dwvInfo[1] == dwP1))
			return pPos-pMap->svColliders.begin();
	}

	return -1;
}

int FindDrop(collision_map_p pMap, int dwP1, int dwP2)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_DROP  &&  (pPos->dwvInfo[0] == dwP1  &&  pPos->dwvInfo[1] == dwP2  ||  pPos->dwvInfo[0] == dwP2  &&  pPos->dwvInfo[1] == dwP1))
			return pPos-pMap->svColliders.begin();
	}

	return -1;
}

int FindKill(collision_map_p pMap, int dwP1, int dwP2)
{
	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_KILL  &&  (pPos->dwvInfo[0] == dwP1  &&  pPos->dwvInfo[1] == dwP2  ||  pPos->dwvInfo[0] == dwP2  &&  pPos->dwvInfo[1] == dwP1))
			return pPos-pMap->svColliders.begin();
	}

	return -1;
}

void AddBlocker(collision_map_p pMap, int dwX1, int dwY1, int dwX2, int dwY2)
{
	int dwP1 = FindColliderPoint(pMap, dwX1, dwY1);
	int dwP2 = FindColliderPoint(pMap, dwX2, dwY2);

	if(dwP1 == dwP2  ||  FindBlocker(pMap, dwP1, dwP2) >= 0)
		return;

	collider_s sCol;
	sCol.dwType = CT_BLOCKER;
	sCol.dwvInfo[0] = dwP1;
	sCol.dwvInfo[1] = dwP2;
	sCol.dwvInfo[2] = 0;
	sCol.dwvInfo[3] = 0;

	sCol.fvDir[0] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[0]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[0];
	sCol.fvDir[1] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[1]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[1];

	sCol.fLength = Vec2Length(sCol.fvDir);
	Vec2Divide(sCol.fvDir, sCol.fLength, sCol.fvDir);

	pMap->svColliders.push_back(sCol);
}

void AddLeftGrab(collision_map_p pMap, int dwX1, int dwY1)
{
	int dwP1 = FindColliderPoint(pMap, dwX1, dwY1);

	if(FindLeftGrab(pMap, dwP1) >= 0)
		return;

	collider_s sCol;
	sCol.dwType = CT_LEFT_GRAB;
	sCol.dwvInfo[0] = dwP1;
	sCol.dwvInfo[1] = 0;
	sCol.dwvInfo[2] = 0;
	sCol.dwvInfo[3] = 0;

	pMap->svColliders.push_back(sCol);
}

void AddRightGrab(collision_map_p pMap, int dwX1, int dwY1)
{
	int dwP1 = FindColliderPoint(pMap, dwX1, dwY1);

	if(FindRightGrab(pMap, dwP1) >= 0)
		return;

	collider_s sCol;
	sCol.dwType = CT_RIGHT_GRAB;
	sCol.dwvInfo[0] = dwP1;
	sCol.dwvInfo[1] = 0;
	sCol.dwvInfo[2] = 0;
	sCol.dwvInfo[3] = 0;

	pMap->svColliders.push_back(sCol);
}

void AddPole(collision_map_p pMap, int dwX1, int dwY1, int dwX2, int dwY2)
{
	int dwP1 = FindColliderPoint(pMap, dwX1, dwY1);
	int dwP2 = FindColliderPoint(pMap, dwX2, dwY2);

	if(dwP1 == dwP2  ||  FindPole(pMap, dwP1, dwP2) >= 0)
		return;

	collider_s sCol;
	sCol.dwType = CT_POLE;
	sCol.dwvInfo[0] = dwP1;
	sCol.dwvInfo[1] = dwP2;
	sCol.dwvInfo[2] = 0;
	sCol.dwvInfo[3] = 0;

	sCol.fvDir[0] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[0]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[0];
	sCol.fvDir[1] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[1]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[1];

	sCol.fLength = Vec2Length(sCol.fvDir);
	Vec2Divide(sCol.fvDir, sCol.fLength, sCol.fvDir);

	pMap->svColliders.push_back(sCol);
}

void AddDrop(collision_map_p pMap, int dwX1, int dwY1, int dwX2, int dwY2)
{
	int dwP1 = FindColliderPoint(pMap, dwX1, dwY1);
	int dwP2 = FindColliderPoint(pMap, dwX2, dwY2);

	if(dwP1 == dwP2  ||  FindDrop(pMap, dwP1, dwP2) >= 0)
		return;

	collider_s sCol;
	sCol.dwType = CT_DROP;
	sCol.dwvInfo[0] = dwP1;
	sCol.dwvInfo[1] = dwP2;
	sCol.dwvInfo[2] = 0;
	sCol.dwvInfo[3] = 0;

	sCol.fvDir[0] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[0]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[0];
	sCol.fvDir[1] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[1]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[1];

	sCol.fLength = Vec2Length(sCol.fvDir);
	Vec2Divide(sCol.fvDir, sCol.fLength, sCol.fvDir);

	pMap->svColliders.push_back(sCol);
}

void AddKill(collision_map_p pMap, int dwX1, int dwY1, int dwX2, int dwY2)
{
	int dwP1 = FindColliderPoint(pMap, dwX1, dwY1);
	int dwP2 = FindColliderPoint(pMap, dwX2, dwY2);

	if(dwP1 == dwP2  ||  FindKill(pMap, dwP1, dwP2) >= 0)
		return;

	collider_s sCol;
	sCol.dwType = CT_KILL;
	sCol.dwvInfo[0] = dwP1;
	sCol.dwvInfo[1] = dwP2;
	sCol.dwvInfo[2] = 0;
	sCol.dwvInfo[3] = 0;

	sCol.fvDir[0] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[0]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[0];
	sCol.fvDir[1] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[1]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[1];

	sCol.fLength = Vec2Length(sCol.fvDir);
	Vec2Divide(sCol.fvDir, sCol.fLength, sCol.fvDir);

	pMap->svColliders.push_back(sCol);
}

void ThickLine(BITMAP* pBmp, int dwX, int dwY, int dwColor)
{
//	rectfill(pBmp, dwX-1, dwY-1, dwX+1, dwY+2, dwColor);
	putpixel(pBmp, dwX, dwY, dwColor);
	putpixel(pBmp, dwX+1, dwY, dwColor);
	putpixel(pBmp, dwX-1, dwY, dwColor);
	putpixel(pBmp, dwX, dwY+1, dwColor);
	putpixel(pBmp, dwX, dwY-1, dwColor);
}

void DrawDLine(BITMAP* pBmp, float fX1, float fY1, float fX2, float fY2, float fPerpSize, int dwColor)
{
	float fvDir[2] = { fY2-fY1, fX1-fX2 };
	Vec2SetLength(fvDir, fPerpSize, fvDir);

	float fX3 = (fX1+fX2)/2.0f;
	float fY3 = (fY1+fY2)/2.0f;

	do_line(pBmp, fX1, fY1, fX2, fY2, dwColor, ThickLine);
//	do_line(pBmp, fX1, fY1, fX3, fY3, dwColor, ThickLine);
//	do_line(pBmp, fX3, fY3, fX2, fY2, makecol(128, 0, 0), ThickLine);
	do_line(pBmp, fX3, fY3, fX3+fvDir[0], fY3+fvDir[1], dwColor, ThickLine);
}

void DrawCollisionMap(BITMAP* pBmp, collision_map_p pMap, int dwX, int dwY)
{
	col_point_p pP1, pP2;
	collider_list_t::iterator pPos;
	for(pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		switch(pPos->dwType)
		{
		case CT_BLOCKER:
			{
				pP1 = pMap->svPoints.begin()+pPos->dwvInfo[0];
				pP2 = pMap->svPoints.begin()+pPos->dwvInfo[1];
				DrawDLine(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP2->dwvPos[0]+dwX, pP2->dwvPos[1]+dwY, 8, makecol(255, 64, 0));
				break;
			}
		case CT_POLE:
			{
				pP1 = pMap->svPoints.begin()+pPos->dwvInfo[0];
				pP2 = pMap->svPoints.begin()+pPos->dwvInfo[1];
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP2->dwvPos[0]+dwX, pP2->dwvPos[1]+dwY, makecol(255, 255, 0), ThickLine);
				break;
			}
		case CT_DROP:
			{
				pP1 = pMap->svPoints.begin()+pPos->dwvInfo[0];
				pP2 = pMap->svPoints.begin()+pPos->dwvInfo[1];
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP2->dwvPos[0]+dwX, pP2->dwvPos[1]+dwY, makecol(255, 0, 255), ThickLine);
				break;
			}
		case CT_KILL:
			{
				pP1 = pMap->svPoints.begin()+pPos->dwvInfo[0];
				pP2 = pMap->svPoints.begin()+pPos->dwvInfo[1];
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP2->dwvPos[0]+dwX, pP2->dwvPos[1]+dwY, makecol(0, 192, 192), ThickLine);
				break;
			}
		}
	}
	for(pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		switch(pPos->dwType)
		{
		case CT_LEFT_GRAB:
			{
				pP1 = pMap->svPoints.begin()+pPos->dwvInfo[0];
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+8+dwY, makecol(0, 255, 0), ThickLine);
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP1->dwvPos[0]+8+dwX, pP1->dwvPos[1]+dwY, makecol(0, 255, 0), ThickLine);
				break;
			}
		case CT_RIGHT_GRAB:
			{
				pP1 = pMap->svPoints.begin()+pPos->dwvInfo[0];
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+8+dwY, makecol(0, 255, 0), ThickLine);
				do_line(pBmp, pP1->dwvPos[0]+dwX, pP1->dwvPos[1]+dwY, pP1->dwvPos[0]-8+dwX, pP1->dwvPos[1]+dwY, makecol(0, 255, 0), ThickLine);
				break;
			}
		}
	}
}

void SaveCollisionMap(string sDir, collision_map_p pMap)
{
	PACKFILE* pFile = pack_fopen((sDir+"ColMap.mpc").c_str(), "w");

	int dwVersion = 1;
	pack_fwrite(&dwVersion, sizeof(int), pFile);

	int dwCount = pMap->svPoints.size();
	pack_fwrite(&dwCount, sizeof(int), pFile);

	dwCount = pMap->svColliders.size();
	pack_fwrite(&dwCount, sizeof(int), pFile);

	for(col_point_list_t::iterator pPoint=pMap->svPoints.begin(); pPoint!=pMap->svPoints.end(); ++pPoint)
	{
		pack_fwrite(&pPoint->dwvPos[0], sizeof(int), pFile);
		pack_fwrite(&pPoint->dwvPos[1], sizeof(int), pFile);
	}

	for(collider_list_t::iterator pCol=pMap->svColliders.begin(); pCol!=pMap->svColliders.end(); ++pCol)
	{
		pack_fwrite(&pCol->dwType, sizeof(int), pFile);
		pack_fwrite(&pCol->dwvInfo[0], sizeof(int), pFile);
		pack_fwrite(&pCol->dwvInfo[1], sizeof(int), pFile);
		pack_fwrite(&pCol->dwvInfo[2], sizeof(int), pFile);
		pack_fwrite(&pCol->dwvInfo[3], sizeof(int), pFile);
	}

	pack_fclose(pFile);
}

collision_map_p LoadCollisionMap(string sDir)
{
	collision_map_p pMap = new collision_map_t;

	PACKFILE* pFile = pack_fopen((sDir+"ColMap.mpc").c_str(), "r");
	if(pFile)
	{
		int dwVersion;
		pack_fread(&dwVersion, sizeof(int), pFile);

		int dwPC = pMap->svPoints.size();
		pack_fread(&dwPC, sizeof(int), pFile);

		int dwCC = pMap->svColliders.size();
		pack_fread(&dwCC, sizeof(int), pFile);

		int lp;
		for(lp=0; lp<dwPC; lp++)
		{
			col_point_t sPoint;

			pack_fread(&sPoint.dwvPos[0], sizeof(int), pFile);
			pack_fread(&sPoint.dwvPos[1], sizeof(int), pFile);

			pMap->svPoints.push_back(sPoint);
		}

		for(lp=0; lp<dwCC; lp++)
		{
			collider_t sCol;

			pack_fread(&sCol.dwType, sizeof(int), pFile);
			pack_fread(&sCol.dwvInfo[0], sizeof(int), pFile);
			pack_fread(&sCol.dwvInfo[1], sizeof(int), pFile);
			pack_fread(&sCol.dwvInfo[2], sizeof(int), pFile);
			pack_fread(&sCol.dwvInfo[3], sizeof(int), pFile);

			if(sCol.dwType != CT_LEFT_GRAB  &&  sCol.dwType != CT_RIGHT_GRAB)
			{
				sCol.fvDir[0] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[0]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[0];
				sCol.fvDir[1] = pMap->svPoints[sCol.dwvInfo[1]].dwvPos[1]-pMap->svPoints[sCol.dwvInfo[0]].dwvPos[1];

				sCol.fLength = Vec2Length(sCol.fvDir);
				Vec2Divide(sCol.fvDir, sCol.fLength, sCol.fvDir);
			}

			pMap->svColliders.push_back(sCol);
		}

		pack_fclose(pFile);
	}

	return pMap;
}

float CheckLine_Blocker(collision_map_p pMap, collider_p pCol, float fX1, float fY1, float& fX2, float& fY2, float fXRadius, float fYRadius, float* fvClip)
{
	if(pCol->dwType != CT_BLOCKER)
		return 1.0f;

	float fvDir[2] = { fX2-fX1, fY2-fY1 };
	float fLength = Vec2Length(fvDir);
	Vec2Divide(fvDir, fLength, fvDir);
	float fMove = fLength;
	if(fLength == 0.0f)
	{
		return 1.0f;
	}

	float fAspect = fXRadius/fYRadius;
	float fvPos[2] = { fX1, fY1*fAspect };
	float fvMoveDir[2] = { fX2-fX1, (fY2-fY1)*fAspect };
	float fDist = Vec2Length(fvMoveDir);
	Vec2Divide(fvMoveDir, fDist, fvMoveDir);

	float fvLinePos[2];
	float fvLineDir[2];
	float fvLinePerp[2];
	float fLineLength;

	float fvTPos[2];
	float fvEPos[2];

	float fIDist, fLDist, fD;
	if((fvDir[0]*pCol->fvDir[1]-fvDir[1]*pCol->fvDir[0]) <= 0.02f)
	{
		fvLinePos[0] = pMap->svPoints[pCol->dwvInfo[0]].dwvPos[0];
		fvLinePos[1] = pMap->svPoints[pCol->dwvInfo[0]].dwvPos[1]*fAspect;
		fvLineDir[0] = pMap->svPoints[pCol->dwvInfo[1]].dwvPos[0]-pMap->svPoints[pCol->dwvInfo[0]].dwvPos[0];
		fvLineDir[1] = (pMap->svPoints[pCol->dwvInfo[1]].dwvPos[1]-pMap->svPoints[pCol->dwvInfo[0]].dwvPos[1])*fAspect;
		fLineLength = Vec2Length(fvLineDir);
		Vec2Divide(fvLineDir, fLineLength, fvLineDir);
		fvLinePerp[0] = fvLineDir[1];
		fvLinePerp[1] = -fvLineDir[0];

		fD = -Vec2Dot(fvLinePos, fvLinePerp)-fXRadius;
//			if(Vec2LineDist(fD, fvLinePerp, fvPos) >= 0.0f)
		{
			fIDist = Vec2IntersectPoint(fD, fvLinePerp, fvPos, fvMoveDir, fvTPos);
			if(fIDist <= fDist)
			{
				fLDist = (fvTPos[0]-fvLinePos[0])*fvLineDir[0]+(fvTPos[1]-fvLinePos[1])*fvLineDir[1];
				if(fLDist <= 0.0f)
				{
					Vec2Copy(fvLinePos, fvEPos);
					fIDist = Vec2IntersectCircle(fvEPos, fXRadius, fvPos, fvMoveDir);
					if(fIDist >= 0.0f  &&  fIDist <= fDist)
					{
						fvClip[0] = (fvPos[0]+fvMoveDir[0]*fIDist)-fvEPos[0];
						fvClip[1] = (fvPos[1]+fvMoveDir[1]*fIDist)-fvEPos[1];
						fvClip[0] /= fAspect;
						fD = Vec2Length(fvClip);
						if(fD == 0.0f)
							fD = 1.0f;
						Vec2Divide(fvClip, fD, fvClip);
						if(ABS(fvClip[0]) > 1.0f)
						{
							fX2 = fX2;
						}
						fX2 = fX1+fvMoveDir[0]*fIDist;
						fY2 = fY1+(fvMoveDir[1]*fIDist)/fAspect;
						fDist = fIDist;
						fMove = fvDir[0]*(fX2-fX1)+fvDir[1]*(fY2-fY1);
					}
				}
				else if(fLDist >= fLineLength)
				{
					fvEPos[0] = (float)pMap->svPoints[pCol->dwvInfo[1]].dwvPos[0];
					fvEPos[1] = (float)pMap->svPoints[pCol->dwvInfo[1]].dwvPos[1]*fAspect;
					fIDist = Vec2IntersectCircle(fvEPos, fXRadius, fvPos, fvMoveDir);
					if(fIDist >= 0.0f  &&  fIDist <= fDist)
					{
						fvClip[0] = (fvPos[0]+fvMoveDir[0]*fIDist)-fvEPos[0];
						fvClip[1] = (fvPos[1]+fvMoveDir[1]*fIDist)-fvEPos[1];
						fvClip[0] /= fAspect;
						fD = Vec2Length(fvClip);
						if(fD == 0.0f)
							fD = 1.0f;
						Vec2Divide(fvClip, fD, fvClip);
						if(ABS(fvClip[0]) > 1.0f)
						{
							fX2 = fX2;
						}
						fX2 = fX1+fvMoveDir[0]*fIDist;
						fY2 = fY1+(fvMoveDir[1]*fIDist)/fAspect;
						fDist = fIDist;
						fMove = fvDir[0]*(fX2-fX1)+fvDir[1]*(fY2-fY1);
					}
				}
				else if(fIDist >= -0.0125f)
				{
					fvClip[0] = pCol->fvDir[1];
					fvClip[1] = -pCol->fvDir[0];
					if(ABS(fvClip[0]) > 1.0f)
					{
						fX2 = fX2;
					}
					fX2 = fvTPos[0];
					fY2 = fvTPos[1]/fAspect;
					fDist = fIDist;
					fMove = fvDir[0]*(fX2-fX1)+fvDir[1]*(fY2-fY1);
				}
			}
		}
	}

	return fMove/fLength;
}

float CheckLine_Blocker(collision_map_p pMap, collision_node_p pNode, float fX1, float fY1, float& fX2, float& fY2, float fXRadius, float fYRadius, float* fvClip)
{
	if(!pNode)
		return 1.0f;

	float fTX, fTY;
	float fOX = fX2;
	float fOY = fY2;
	float fvTClip[2] = { 0.0f, 0.0f };
	float fTMove = 1.0f;
	float fMove = 1.0f;

	for(collider_list_pt::iterator pPos=pNode->pvColliders.begin(); pPos!=pNode->pvColliders.end(); ++pPos)
	{
		fTX = fOX;
		fTY = fOY;
		fTMove = CheckLine_Blocker(pMap, *pPos, fX1, fY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
		if(fTMove < fMove)
		{
			fMove = fTMove;
			fvClip[0] = fvTClip[0];
			fvClip[1] = fvTClip[1];
			fX2 = fTX;
			fY2 = fTY;
		}
	}

	if(fX1-fXRadius <= pNode->dwvCutPoint[0]  ||  fOX-fXRadius <= pNode->dwvCutPoint[0])
	{
		if(fY1-fYRadius <= pNode->dwvCutPoint[1]  ||  fOY-fYRadius <= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = CheckLine_Blocker(pMap, pNode->pvChildren[0], fX1, fY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
		if(fY1+fYRadius >= pNode->dwvCutPoint[1]  ||  fOY+fYRadius >= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = CheckLine_Blocker(pMap, pNode->pvChildren[2], fX1, fY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
	}
	if(fX1+fXRadius >= pNode->dwvCutPoint[0]  ||  fOX+fXRadius >= pNode->dwvCutPoint[0])
	{
		if(fY1-fYRadius <= pNode->dwvCutPoint[1]  ||  fOY-fYRadius <= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = CheckLine_Blocker(pMap, pNode->pvChildren[1], fX1, fY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
		if(fY1+fYRadius >= pNode->dwvCutPoint[1]  ||  fOY+fYRadius >= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = CheckLine_Blocker(pMap, pNode->pvChildren[3], fX1, fY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
	}

	return fMove;
}

float CheckLine_Blocker(collision_quadmap_p pQuadMap, float fX1, float fY1, float& fX2, float& fY2, float fXRadius, float fYRadius, float fTime, float* fvClip)
{
	float fTX, fTY;
	float fOX = fX2;
	float fOY = fY2;
	float fvTClip[2] = { 0.0f, 0.0f };
	float fTMove = 1.0f;
	float fMove = 1.0f;

	fTX = fOX;
	fTY = fOY;
	fTMove = CheckLine_Blocker(pQuadMap->pBaseMap, pQuadMap->pBaseNode, fX1, fY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
	if(fTMove < fMove)
	{
		fMove = fTMove;
		fvClip[0] = fvTClip[0];
		fvClip[1] = fvTClip[1];
		fX2 = fTX;
		fY2 = fTY;
	}

	float fTX1, fTY1;
	for(platform_list_t::iterator pPos=pQuadMap->pvPlatforms.begin(); pPos!=pQuadMap->pvPlatforms.end(); ++pPos)
	{
		fTX1 = fX1-(*pPos)->fvMove[0]*fTime-(*pPos)->fvPos[0];
		fTY1 = fY1-(*pPos)->fvMove[1]*fTime-(*pPos)->fvPos[1];
		for(collider_list_t::iterator pCol=(*pPos)->sMap.svColliders.begin(); pCol!=(*pPos)->sMap.svColliders.end(); ++pCol)
		{
			fTX = fOX-(*pPos)->fvMove[0]-(*pPos)->fvPos[0];
			fTY = fOY-(*pPos)->fvMove[1]-(*pPos)->fvPos[1];
			fTMove = CheckLine_Blocker(&(*pPos)->sMap, pCol, fTX1, fTY1, fTX, fTY, fXRadius, fYRadius, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				Vec2Multiply((*pPos)->fvMove, 100.0f, &fvClip[2]);
//				fT = (1.0f-fTime)*fMove+fTime;
				fX2 = fTX+(*pPos)->fvMove[0]+(*pPos)->fvPos[0];
				fY2 = fTY+(*pPos)->fvMove[1]+(*pPos)->fvPos[1];
			}
		}
	}

	return fMove;
}

float TraceLine_Blocker(collision_map_p pMap, collider_p pCol, float fX1, float fY1, float& fX2, float& fY2, float* fvClip)
{
	if(pCol->dwType != CT_BLOCKER)
		return 1.0f;

	float fvDir[2] = { fX2-fX1, fY2-fY1 };
	float fLength = Vec2Length(fvDir);
	Vec2Divide(fvDir, fLength, fvDir);
	float fMove = fLength;
	float fvPos[2] = { fX1, fY1 };
	if(fLength == 0.0f)
	{
		return 1.0f;
	}

	float fvLinePos[2];
	float fvLinePerp[2];

	float fvTPos[2];
	float fIDist, fLDist;
	if((fvDir[0]*pCol->fvDir[1]-fvDir[1]*pCol->fvDir[0]) <= 0.0f)
	{
		fvLinePos[0] = pMap->svPoints[pCol->dwvInfo[0]].dwvPos[0];
		fvLinePos[1] = pMap->svPoints[pCol->dwvInfo[0]].dwvPos[1];
		fvLinePerp[0] = pCol->fvDir[1];
		fvLinePerp[1] = -pCol->fvDir[0];

		fIDist = Vec2IntersectPoint(-Vec2Dot(fvLinePos, fvLinePerp), fvLinePerp, fvPos, fvDir, fvTPos);
		if(fIDist >= 0.0f  &&  fIDist <= fMove)
		{
			fLDist = (fvTPos[0]-fvLinePos[0])*pCol->fvDir[0]+(fvTPos[1]-fvLinePos[1])*pCol->fvDir[1];
			if(fLDist >= 0.0f  &&  fLDist <= pCol->fLength)
			{
				fvClip[0] = pCol->fvDir[1];
				fvClip[1] = -pCol->fvDir[0];
				fX2 = fvTPos[0];
				fY2 = fvTPos[1];
				fMove = fIDist;
			}
		}
	}

	return fMove/fLength;
}

float TraceLine_Blocker(collision_map_p pMap, collision_node_p pNode, float fX1, float fY1, float& fX2, float& fY2, float* fvClip)
{
	if(!pNode)
		return 1.0f;

	float fTX, fTY;
	float fOX = fX2;
	float fOY = fY2;
	float fvTClip[2] = { 0.0f, 0.0f };
	float fTMove = 1.0f;
	float fMove = 1.0f;

	for(collider_list_pt::iterator pPos=pNode->pvColliders.begin(); pPos!=pNode->pvColliders.end(); ++pPos)
	{
		fTX = fOX;
		fTY = fOY;
		fTMove = TraceLine_Blocker(pMap, *pPos, fX1, fY1, fTX, fTY, fvTClip);
		if(fTMove < fMove)
		{
			fMove = fTMove;
			fvClip[0] = fvTClip[0];
			fvClip[1] = fvTClip[1];
			fX2 = fTX;
			fY2 = fTY;
		}
	}

	if(fX1 <= pNode->dwvCutPoint[0]  ||  fOX <= pNode->dwvCutPoint[0])
	{
		if(fY1 <= pNode->dwvCutPoint[1]  ||  fOY <= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = TraceLine_Blocker(pMap, pNode->pvChildren[0], fX1, fY1, fTX, fTY, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
		if(fY1 >= pNode->dwvCutPoint[1]  ||  fOY >= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = TraceLine_Blocker(pMap, pNode->pvChildren[2], fX1, fY1, fTX, fTY, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
	}
	if(fX1 >= pNode->dwvCutPoint[0]  ||  fOX >= pNode->dwvCutPoint[0])
	{
		if(fY1 <= pNode->dwvCutPoint[1]  ||  fOY <= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = TraceLine_Blocker(pMap, pNode->pvChildren[1], fX1, fY1, fTX, fTY, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
		if(fY1 >= pNode->dwvCutPoint[1]  ||  fOY >= pNode->dwvCutPoint[1])
		{
			fTX = fOX;
			fTY = fOY;
			fTMove = TraceLine_Blocker(pMap, pNode->pvChildren[3], fX1, fY1, fTX, fTY, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX;
				fY2 = fTY;
			}
		}
	}

	return fMove;
}

float TraceLine_Blocker(collision_quadmap_p pQuadMap, float fX1, float fY1, float& fX2, float& fY2, float* fvClip)
{
	float fTX, fTY;
	float fOX = fX2;
	float fOY = fY2;
	float fvTClip[2] = { 0.0f, 0.0f };
	float fTMove = 1.0f;
	float fMove = 1.0f;

	fTX = fOX;
	fTY = fOY;
	fTMove = TraceLine_Blocker(pQuadMap->pBaseMap, pQuadMap->pBaseNode, fX1, fY1, fTX, fTY, fvTClip);
	if(fTMove < fMove)
	{
		fMove = fTMove;
		fvClip[0] = fvTClip[0];
		fvClip[1] = fvTClip[1];
		fX2 = fTX;
		fY2 = fTY;
	}

	float fTX1, fTY1;
	for(platform_list_t::iterator pPos=pQuadMap->pvPlatforms.begin(); pPos!=pQuadMap->pvPlatforms.end(); ++pPos)
	{
		fTX1 = fX1-(*pPos)->fvPos[0];
		fTY1 = fY1-(*pPos)->fvPos[1];
		for(collider_list_t::iterator pCol=(*pPos)->sMap.svColliders.begin(); pCol!=(*pPos)->sMap.svColliders.end(); ++pCol)
		{
			fTX = fOX-(*pPos)->fvMove[0]-(*pPos)->fvPos[0];
			fTY = fOY-(*pPos)->fvMove[1]-(*pPos)->fvPos[1];
			fTMove = TraceLine_Blocker(&(*pPos)->sMap, pCol, fTX1, fTY1, fTX, fTY, fvTClip);
			if(fTMove < fMove)
			{
				fMove = fTMove;
				fvClip[0] = fvTClip[0];
				fvClip[1] = fvTClip[1];
				fX2 = fTX+(*pPos)->fvMove[0]+(*pPos)->fvPos[0];
				fY2 = fTY+(*pPos)->fvMove[1]+(*pPos)->fvPos[1];
			}
		}
	}

	return fMove;
}

int NearestLeftGrab(collision_map_p pMap, int dwX, int dwY, int dwRange)
{
	col_point_p pPnt;
	for(collider_list_t::iterator pCol=pMap->svColliders.begin(); pCol!=pMap->svColliders.end(); ++pCol)
	{
		if(pCol->dwType == CT_LEFT_GRAB)
		{
			pPnt = pMap->svPoints.begin()+pCol->dwvInfo[0];
			if(ABS(pPnt->dwvPos[0]-dwX) <= dwRange  &&  ABS(pPnt->dwvPos[1]-dwY) <= dwRange)
			{
				return pCol-pMap->svColliders.begin();
			}
		}
	}

	return -1;
}

int NearestRightGrab(collision_map_p pMap, int dwX, int dwY, int dwRange)
{
	col_point_p pPnt;
	for(collider_list_t::iterator pCol=pMap->svColliders.begin(); pCol!=pMap->svColliders.end(); ++pCol)
	{
		if(pCol->dwType == CT_RIGHT_GRAB)
		{
			pPnt = pMap->svPoints.begin()+pCol->dwvInfo[0];
			if(ABS(pPnt->dwvPos[0]-dwX) <= dwRange  &&  ABS(pPnt->dwvPos[1]-dwY) <= dwRange)
			{
				return pCol-pMap->svColliders.begin();
			}
		}
	}

	return -1;
}

int NearestBlocker(collision_map_p pMap, int dwX, int dwY, int dwRange)
{
	int dwCol = -1;
	float fDist = dwRange;

	float fD, fLD;
	float fvDiff[2];
	float fvPerp[2];
	col_point_p pPnt;
	for(collider_list_t::iterator pCol=pMap->svColliders.begin(); pCol!=pMap->svColliders.end(); ++pCol)
	{
		if(pCol->dwType == CT_BLOCKER)
		{
			pPnt = pMap->svPoints.begin()+pCol->dwvInfo[0];

			fvDiff[0] = dwX-pPnt->dwvPos[0];
			fvDiff[1] = dwY-pPnt->dwvPos[1];

			fvPerp[0] = pCol->fvDir[1];
			fvPerp[1] = -pCol->fvDir[0];
			fLD = Vec2LineDist(0.0f, fvPerp, fvDiff);
			if(fLD > 0.0f  &&  fLD <= fDist)
			{
				fD = Vec2Dot(fvDiff, pCol->fvDir);
				if(fD < 0.0f)
				{
					if((fD = Vec2Length(fvDiff)) <= fDist)
					{
						fDist = fD;
						dwCol = pCol-pMap->svColliders.begin();
					}
				}
				else if(fD > pCol->fLength)
				{
					pPnt = pMap->svPoints.begin()+pCol->dwvInfo[1];
					fvDiff[0] = dwX-pPnt->dwvPos[0];
					fvDiff[1] = dwY-pPnt->dwvPos[1];
					if((fD = Vec2Length(fvDiff)) <= fDist)
					{
						fDist = fD;
						dwCol = pCol-pMap->svColliders.begin();
					}
				}
				else
				{
					fDist = fLD;
					dwCol = pCol-pMap->svColliders.begin();
				}
			}
		}
	}

	return dwCol;
}

int NearestLine(collision_map_p pMap, int dwX, int dwY, int dwRange, int dwType)
{
	int dwCol = -1;
	float fDist = dwRange;

	float fD;
	float fvDiff[2];
	float fvPerp[2];
	col_point_p pPnt;
	for(collider_list_t::iterator pCol=pMap->svColliders.begin(); pCol!=pMap->svColliders.end(); ++pCol)
	{
		if(pCol->dwType == dwType)
		{
			pPnt = pMap->svPoints.begin()+pCol->dwvInfo[0];

			fvDiff[0] = dwX-pPnt->dwvPos[0];
			fvDiff[1] = dwY-pPnt->dwvPos[1];
			fD = Vec2Dot(fvDiff, pCol->fvDir);
			if(fD < 0.0f)
			{
				if((fD = Vec2Length(fvDiff)) <= fDist)
				{
					fDist = fD;
					dwCol = pCol-pMap->svColliders.begin();
				}
			}
			else if(fD > pCol->fLength)
			{
				pPnt = pMap->svPoints.begin()+pCol->dwvInfo[1];
				fvDiff[0] = dwX-pPnt->dwvPos[0];
				fvDiff[1] = dwY-pPnt->dwvPos[1];
				if((fD = Vec2Length(fvDiff)) <= fDist)
				{
					fDist = fD;
					dwCol = pCol-pMap->svColliders.begin();
				}
			}
			else
			{
				fvPerp[0] = pCol->fvDir[1];
				fvPerp[1] = -pCol->fvDir[0];
				fD = Vec2LineDist(0.0f, fvPerp, fvDiff);
				fD = ABS(fD);
				if(fD <= fDist)
				{
					fDist = fD;
					dwCol = pCol-pMap->svColliders.begin();
				}
			}
		}
	}

	return dwCol;
}

int NearestPole(collision_map_p pMap, int dwX, int dwY, int dwRange)
{
	return NearestLine(pMap, dwX, dwY, dwRange, CT_POLE);
}

int NearestDrop(collision_map_p pMap, int dwX, int dwY, int dwRange)
{
	return NearestLine(pMap, dwX, dwY, dwRange, CT_DROP);
}

int NearestKill(collision_map_p pMap, int dwX, int dwY, int dwRange)
{
	return NearestLine(pMap, dwX, dwY, dwRange, CT_KILL);
}

void DeleteCollider(collision_map_p pMap, int dwIndex)
{
	pMap->svColliders[dwIndex].dwType = CT_NONE;
}

void AddColliderToNode(collision_node_p& pNode, collider_p pCollider, int dwX1, int dwY1, int dwX2, int dwY2, col_point_list_t& svPoints, int dwMinSize)
{
	if(!pNode)
	{
		pNode = new collision_node_t;
		pNode->dwvCutPoint[0] = (dwX1+dwX2)>>1;
		pNode->dwvCutPoint[1] = (dwY1+dwY2)>>1;
		pNode->pvChildren[0] = NULL;
		pNode->pvChildren[1] = NULL;
		pNode->pvChildren[2] = NULL;
		pNode->pvChildren[3] = NULL;
		pNode->pvColliders.clear();
	}

	int dwNode = -1;
	if((dwX2-dwX1) > dwMinSize  &&  (dwY2-dwY1) > dwMinSize)
	{
		if(svPoints[pCollider->dwvInfo[0]].dwvPos[0] <= pNode->dwvCutPoint[0]  &&  svPoints[pCollider->dwvInfo[1]].dwvPos[0] <= pNode->dwvCutPoint[0])
		{
			if(svPoints[pCollider->dwvInfo[0]].dwvPos[1] <= pNode->dwvCutPoint[1]  &&  svPoints[pCollider->dwvInfo[1]].dwvPos[1] <= pNode->dwvCutPoint[1])
			{
				dwNode = 0;
			}
			else if(svPoints[pCollider->dwvInfo[0]].dwvPos[1] >= pNode->dwvCutPoint[1]  &&  svPoints[pCollider->dwvInfo[1]].dwvPos[1] >= pNode->dwvCutPoint[1])
			{
				dwNode = 2;
			}
		}
		else if(svPoints[pCollider->dwvInfo[0]].dwvPos[0] >= pNode->dwvCutPoint[0]  &&  svPoints[pCollider->dwvInfo[1]].dwvPos[0] >= pNode->dwvCutPoint[0])
		{
			if(svPoints[pCollider->dwvInfo[0]].dwvPos[1] <= pNode->dwvCutPoint[1]  &&  svPoints[pCollider->dwvInfo[1]].dwvPos[1] <= pNode->dwvCutPoint[1])
			{
				dwNode = 1;
			}
			else if(svPoints[pCollider->dwvInfo[0]].dwvPos[1] >= pNode->dwvCutPoint[1]  &&  svPoints[pCollider->dwvInfo[1]].dwvPos[1] >= pNode->dwvCutPoint[1])
			{
				dwNode = 3;
			}
		}
	}

	int dwMX = (dwX1+dwX2)>>1;
	int dwMY = (dwY1+dwY2)>>1;
	switch(dwNode)
	{
	case 0:
		{
			AddColliderToNode(pNode->pvChildren[0], pCollider, dwX1, dwY1, dwMX, dwMY, svPoints, dwMinSize);
			break;
		}
	case 1:
		{
			AddColliderToNode(pNode->pvChildren[1], pCollider, dwMX, dwY1, dwX2, dwMY, svPoints, dwMinSize);
			break;
		}
	case 2:
		{
			AddColliderToNode(pNode->pvChildren[2], pCollider, dwX1, dwMY, dwMX, dwY2, svPoints, dwMinSize);
			break;
		}
	case 3:
		{
			AddColliderToNode(pNode->pvChildren[3], pCollider, dwMX, dwMY, dwX2, dwY2, svPoints, dwMinSize);
			break;
		}
	default:
		{
			pNode->pvColliders.push_back(pCollider);
		}
	}
}

collision_quadmap_p BuildCollisionQuadMap(collision_map_p pMap, int dwWidth, int dwHeight, int dwMinSize)
{
	collision_quadmap_p pQuadMap = new collision_quadmap_t;
	pQuadMap->pBaseMap = pMap;
	pQuadMap->pBaseNode = NULL;
	pQuadMap->pvPlatforms.clear();

	for(collider_list_t::iterator pPos=pMap->svColliders.begin(); pPos!=pMap->svColliders.end(); ++pPos)
	{
		if(pPos->dwType == CT_LEFT_GRAB  ||  pPos->dwType == CT_RIGHT_GRAB)
			pPos->dwvInfo[1] = pPos->dwvInfo[0];
		AddColliderToNode(pQuadMap->pBaseNode, pPos, 0, 0, dwWidth, dwHeight, pMap->svPoints, dwMinSize);
	}

	return pQuadMap;
}

void DeletePlatform(platform_p pPlatform)
{
	collision_map_p pMap = &pPlatform->sMap;
	DeleteCollisionMap(pMap);

	delete pPlatform;
}

void DeleteCollisionNode(collision_node_p& pNode)
{
	if(!pNode)
		return;

	pNode->pvColliders.clear();

	DeleteCollisionNode(pNode->pvChildren[0]);
	DeleteCollisionNode(pNode->pvChildren[1]);
	DeleteCollisionNode(pNode->pvChildren[2]);
	DeleteCollisionNode(pNode->pvChildren[3]);

	delete pNode;

	pNode = NULL;
}

void DeleteCollisionQuadMap(collision_quadmap_p& pMap)
{
	DeleteCollisionNode(pMap->pBaseNode);

	for(platform_list_t::iterator pPos=pMap->pvPlatforms.begin(); pPos!=pMap->pvPlatforms.end(); ++pPos)
	{
		DeletePlatform(*pPos);
	}
	pMap->pvPlatforms.clear();

	delete pMap;
}

platform_p CreatePlatform(collision_quadmap_p pQuadMap, entity_p pEntity)
{
	platform_p pPlatform = new platform_t;
	pPlatform->pEntity = pEntity;
	pPlatform->fvPos[0] = pEntity->fvPos[0];
	pPlatform->fvPos[1] = pEntity->fvPos[1];
	pPlatform->fvMove[0] = 0.0f;
	pPlatform->fvMove[1] = 0.0f;
	pPlatform->sMap.svColliders.clear();
	pPlatform->sMap.svPoints.clear();

	pQuadMap->pvPlatforms.push_back(pPlatform);

	return pPlatform;
}

void RunPlatforms(collision_quadmap_p pQuadMap)
{
	for(platform_list_t::iterator pPos=pQuadMap->pvPlatforms.begin(); pPos!=pQuadMap->pvPlatforms.end(); ++pPos)
	{
		Vec2Add((*pPos)->fvPos, (*pPos)->fvMove, (*pPos)->fvPos);
		Vec2Subtract((*pPos)->pEntity->fvPos, (*pPos)->fvPos, (*pPos)->fvMove);
	}
}

float DistToLine(collision_map_p pMap, collider_p pCol, float fX, float fY)
{
	col_point_p pPnt = pMap->svPoints.begin()+pCol->dwvInfo[0];

	float fvDiff[2];
	fvDiff[0] = fX-pPnt->dwvPos[0];
	fvDiff[1] = fY-pPnt->dwvPos[1];
	float fD = Vec2Dot(fvDiff, pCol->fvDir);
	if(fD < 0.0f)
	{
		return Vec2Length(fvDiff);
	}
	else if(fD > pCol->fLength)
	{
		pPnt = pMap->svPoints.begin()+pCol->dwvInfo[1];
		fvDiff[0] = fX-pPnt->dwvPos[0];
		fvDiff[1] = fY-pPnt->dwvPos[1];
		return Vec2Length(fvDiff);
	}
	else
	{
		float fvPerp[2] = { pCol->fvDir[1], -pCol->fvDir[0] };
		fD = Vec2Dot(fvPerp, fvDiff);
		return ABS(fD);
	}
}

collider_p NearestLine(collision_map_p pMap, collision_node_p pNode, float fX, float fY, int dwType, float& fRange)
{
	if(!pNode)
		return NULL;

	collider_p pRet = NULL;
	collider_p pTemp;
	float fD;
	for(collider_list_pt::iterator pCol=pNode->pvColliders.begin(); pCol!=pNode->pvColliders.end(); ++pCol)
	{
		if((*pCol)->dwType == dwType)
		{
			fD = DistToLine(pMap, *pCol, fX, fY);
			if(fD < fRange)
			{
				fRange = fD;
				pRet = *pCol;
			}
		}
	}

	if(fX-fRange <= pNode->dwvCutPoint[0])
	{
		if(fY-fRange <= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestLine(pMap, pNode->pvChildren[0], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
		if(fY+fRange >= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestLine(pMap, pNode->pvChildren[2], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
	}
	if(fX+fRange >= pNode->dwvCutPoint[0])
	{
		if(fY-fRange <= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestLine(pMap, pNode->pvChildren[1], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
		if(fY+fRange >= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestLine(pMap, pNode->pvChildren[3], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
	}

	return pRet;
}

collider_p NearestLine(collision_quadmap_p pQuadMap, float fX, float fY, int dwType, float fRange, platform_p& pPF)
{
	pPF = NULL;
	collider_p pRet = NULL;
	collider_p pTemp;
	if(pTemp = NearestLine(pQuadMap->pBaseMap, pQuadMap->pBaseNode, fX, fY, dwType, fRange))
	{
		pRet = pTemp;
	}

	float fD;
	for(platform_list_t::iterator pPos=pQuadMap->pvPlatforms.begin(); pPos!=pQuadMap->pvPlatforms.end(); ++pPos)
	{
		for(collider_list_t::iterator pCol=(*pPos)->sMap.svColliders.begin(); pCol!=(*pPos)->sMap.svColliders.end(); ++pCol)
		{
			if(pCol->dwType == dwType)
			{
				fD = DistToLine(&(*pPos)->sMap, pCol, fX-(*pPos)->fvPos[0], fY-(*pPos)->fvPos[1]);
				if(fD < fRange)
				{
					fRange = fD;
					pRet = pCol;
					pPF = *pPos;
				}
			}
		}
	}

	return pRet;
}

float DistToGrab(collision_map_p pMap, collider_p pCol, float fX, float fY)
{
	float fvDiff[2] = { fX-pMap->svPoints[pCol->dwvInfo[0]].dwvPos[0], fY-pMap->svPoints[pCol->dwvInfo[0]].dwvPos[1] };
	return Vec2Length(fvDiff);
}

collider_p NearestGrab(collision_map_p pMap, collision_node_p pNode, float fX, float fY, int dwType, float& fRange)
{
	if(!pNode)
		return NULL;

	collider_p pRet = NULL;
	collider_p pTemp;
	float fD;
	for(collider_list_pt::iterator pCol=pNode->pvColliders.begin(); pCol!=pNode->pvColliders.end(); ++pCol)
	{
		if((*pCol)->dwType == dwType)
		{
			fD = DistToGrab(pMap, *pCol, fX, fY);
			if(fD < fRange)
			{
				fRange = fD;
				pRet = *pCol;
			}
		}
	}

	if(fX-fRange <= pNode->dwvCutPoint[0])
	{
		if(fY-fRange <= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestGrab(pMap, pNode->pvChildren[0], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
		if(fY+fRange >= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestGrab(pMap, pNode->pvChildren[2], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
	}
	if(fX+fRange >= pNode->dwvCutPoint[0])
	{
		if(fY-fRange <= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestGrab(pMap, pNode->pvChildren[1], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
		if(fY+fRange >= pNode->dwvCutPoint[1])
		{
			if(pTemp = NearestGrab(pMap, pNode->pvChildren[3], fX, fY, dwType, fRange))
			{
				pRet = pTemp;
			}
		}
	}

	return pRet;
}

collider_p NearestGrab(collision_quadmap_p pQuadMap, float fX, float fY, bool bLeft, bool bRight, float fRange, platform_p& pPF)
{
	collider_p pRet = NULL;
	collider_p pTemp;
	if(bLeft)
	{
		if(pTemp = NearestGrab(pQuadMap->pBaseMap, pQuadMap->pBaseNode, fX, fY, CT_LEFT_GRAB, fRange))
			pRet = pTemp;
	}
	if(bRight)
	{
		if(pTemp = NearestGrab(pQuadMap->pBaseMap, pQuadMap->pBaseNode, fX, fY, CT_RIGHT_GRAB, fRange))
			pRet = pTemp;
	}

	float fD;
	for(platform_list_t::iterator pPos=pQuadMap->pvPlatforms.begin(); pPos!=pQuadMap->pvPlatforms.end(); ++pPos)
	{
		for(collider_list_t::iterator pCol=(*pPos)->sMap.svColliders.begin(); pCol!=(*pPos)->sMap.svColliders.end(); ++pCol)
		{
			if(bLeft  &&  pCol->dwType == CT_LEFT_GRAB)
			{
				fD = DistToGrab(&(*pPos)->sMap, pCol, fX-(*pPos)->fvPos[0], fY-(*pPos)->fvPos[1]);
				if(fD < fRange)
				{
					fRange = fD;
					pRet = pCol;
					pPF = *pPos;
				}
			}
			if(bRight  &&  pCol->dwType == CT_RIGHT_GRAB)
			{
				fD = DistToGrab(&(*pPos)->sMap, pCol, fX-(*pPos)->fvPos[0], fY-(*pPos)->fvPos[1]);
				if(fD < fRange)
				{
					fRange = fD;
					pRet = pCol;
					pPF = *pPos;
				}
			}
		}
	}

	return pRet;
}
