#ifndef COLLIDE_H
#define COLLIDE_H

#include <string>
#include <vector>

using namespace std;

#define CT_NONE				-1
#define CT_BLOCKER			0
#define CT_LEFT_GRAB		1
#define CT_RIGHT_GRAB		2
#define CT_POLE				3
#define CT_DROP				4
#define CT_KILL				5

typedef struct col_point_s {
	int						dwvPos[2];
} col_point_t, *col_point_p;

typedef struct collider_s {
	int						dwType;
	int						dwvInfo[4];
	float					fvDir[2];
	float					fLength;
} collider_t, *collider_p;

typedef vector<col_point_t> col_point_list_t;
typedef vector<collider_t> collider_list_t;
typedef vector<collider_p> collider_list_pt;

typedef struct collision_map_s {
	col_point_list_t		svPoints;
	collider_list_t			svColliders;
} collision_map_t, *collision_map_p;

typedef struct platform_s {
	float					fvPos[2];
	float					fvMove[2];
	struct entity_s*		pEntity;
	collision_map_t			sMap;
} platform_t, *platform_p;
typedef vector<platform_p> platform_list_t;

typedef struct collision_node_s {
	int						dwvCutPoint[2];
	collision_node_s*		pvChildren[4];
	collider_list_pt		pvColliders;
} collision_node_t, *collision_node_p;

typedef struct collision_quadmap_s {
	collision_map_p			pBaseMap;
	collision_node_p		pBaseNode;
	platform_list_t			pvPlatforms;
} collision_quadmap_t, *collision_quadmap_p;

collision_map_p CreateCollisionMap();
void AddBlocker(collision_map_p pMap, int dwX1, int dwX2, int dwY1, int dwY2);
void AddLeftGrab(collision_map_p pMap, int dwX1, int dwY1);
void AddRightGrab(collision_map_p pMap, int dwX1, int dwY1);
void AddPole(collision_map_p pMap, int dwX1, int dwX2, int dwY1, int dwY2);
void AddDrop(collision_map_p pMap, int dwX1, int dwX2, int dwY1, int dwY2);
void AddKill(collision_map_p pMap, int dwX1, int dwX2, int dwY1, int dwY2);
void DrawCollisionMap(BITMAP* pBmp, collision_map_p pMap, int dwX, int dwY);
void SaveCollisionMap(string sDir, collision_map_p pMap);
collision_map_p LoadCollisionMap(string sDir);
float CheckLine_Blocker(collision_quadmap_p pQuadMap, float fX1, float fY1, float& fX2, float& fY2, float fXRadius, float fYRadius, float fTime, float* fvClip);
float TraceLine_Blocker(collision_quadmap_p pQuadMap, float fX1, float fY1, float& fX2, float& fY2, float* fvClip);
int NearestBlocker(collision_map_p pMap, int dwX, int dwY, int dwRange);
int NearestLeftGrab(collision_map_p pMap, int dwX, int dwY, int dwRange);
int NearestRightGrab(collision_map_p pMap, int dwX, int dwY, int dwRange);
int NearestPole(collision_map_p pMap, int dwX, int dwY, int dwRange);
int NearestDrop(collision_map_p pMap, int dwX, int dwY, int dwRange);
int NearestKill(collision_map_p pMap, int dwX, int dwY, int dwRange);
void DeleteCollider(collision_map_p pMap, int dwIndex);
collision_quadmap_p BuildCollisionQuadMap(collision_map_p pMap, int dwWidth, int dwHeight, int dwMinSize);
platform_p CreatePlatform(collision_quadmap_p pQuadMap, struct entity_s* pEntity);
void RunPlatforms(collision_quadmap_p pQuadMap);
collider_p NearestLine(collision_quadmap_p pQuadMap, float fX, float fY, int dwType, float fRange, platform_p& pPF);
collider_p NearestGrab(collision_quadmap_p pQuadMap, float fX, float fY, bool bLeft, bool bRight, float fRange, platform_p& pPF);
void DeleteCollisionMap(collision_map_p& pMap);
void DeleteCollisionQuadMap(collision_quadmap_p& pMap);

#endif // COLLIDE_H
