#include <allegro.h>
#include <winalleg.h>
#include "MapEnts.h"
#include "Vectors.h"

map_ent_type_p FindEntType(map_ent_type_list_t& svEntType, string sName)
{
	for(map_ent_type_list_t::iterator pPos=svEntType.begin(); pPos!=svEntType.end(); ++pPos)
	{
		if((*pPos)->szName == sName)
			return *pPos;
	}

	return NULL;
}

int FindEntTypeIndex(map_ent_type_list_t& svEntType, string sName)
{
	for(map_ent_type_list_t::iterator pPos=svEntType.begin(); pPos!=svEntType.end(); ++pPos)
	{
		if((*pPos)->szName == sName)
			return pPos-svEntType.begin();
	}

	return -1;
}

bool EntNameExists(map_entity_list_t& svEnts, string sName)
{
	for(map_entity_list_t::iterator pEnt=svEnts.begin(); pEnt!=svEnts.end(); ++pEnt)
	{
		if((*pEnt)->sName == sName)
			return true;
	}

	return false;
}

map_entity_p NewMapEntity(map_entity_list_t& svEnts, map_ent_type_p pType, int dwX, int dwY)
{
	map_entity_p pEnt = new map_entity_t;

	int dwNum = 0;
	char szNumber[5];
	do {
		sprintf(szNumber, "%03d", dwNum);
		pEnt->sName = (string)pType->szName+"_"+szNumber;
		dwNum++;
	} while(EntNameExists(svEnts, pEnt->sName));

	pEnt->pType = pType;
	pEnt->dwvPos[0] = dwX-(pType->dwvSize[0]>>1);
	pEnt->dwvPos[1] = dwY-(pType->dwvSize[1]>>1);
	pEnt->dwvPos[2] = pEnt->dwvPos[0]+pType->dwvSize[0]-1;
	pEnt->dwvPos[3] = pEnt->dwvPos[1]+pType->dwvSize[1]-1;
	pEnt->pvPaths[0] = NULL;
	pEnt->pvPaths[1] = NULL;

	for(int lp=0; lp<8; lp++)
		pEnt->svVars[lp] = pType->svVarDef[lp]?pType->svVarDef[lp]:"";

	svEnts.push_back(pEnt);

	return pEnt;
}

void DrawArrow(BITMAP* pBmp, float fX1, float fY1, float fX2, float fY2, float fHeadSize, int dwColor)
{
	float fvDir[2] = { fX2-fX1, fY2-fY1 };
	if(fvDir[0] != 0.0f  ||  fvDir[1] != 0.0f)
	{
		Vec2SetLength(fvDir, fHeadSize, fvDir);
		line(pBmp, fX1, fY1, fX2, fY2, dwColor);
		line(pBmp, fX2, fY2, fX2-fvDir[0]+fvDir[1], fY2-fvDir[1]-fvDir[0], dwColor);
		line(pBmp, fX2, fY2, fX2-fvDir[0]-fvDir[1], fY2-fvDir[1]+fvDir[0], dwColor);
	}
	else
	{
		hline(pBmp, fX1-fHeadSize, fY1, fX1+fHeadSize, dwColor);
		vline(pBmp, fX1, fY1-fHeadSize, fY1+fHeadSize, dwColor);
	}
}

void DrawMapEntPaths(BITMAP* pBmp, map_entity_list_t& svEnts, int dwX, int dwY)
{
	int dwR = rand()%128;
	for(map_entity_list_t::iterator pEnt=svEnts.begin(); pEnt!=svEnts.end(); ++pEnt)
	{
		if((*pEnt)->pvPaths[0])
		{
			DrawArrow(pBmp, 
				(((*pEnt)->dwvPos[0]+(*pEnt)->dwvPos[2])>>1)-dwX, (((*pEnt)->dwvPos[1]+(*pEnt)->dwvPos[3])>>1)-dwY,
				(((*pEnt)->pvPaths[0]->dwvPos[0]+(*pEnt)->pvPaths[0]->dwvPos[2])>>1)-dwX, (((*pEnt)->pvPaths[0]->dwvPos[1]+(*pEnt)->pvPaths[0]->dwvPos[3])>>1)-dwY,
				8, makecol(dwR+64, dwR+64, 255));
		}
		if((*pEnt)->pvPaths[1])
		{
			DrawArrow(pBmp, 
				(((*pEnt)->dwvPos[0]+(*pEnt)->dwvPos[2])>>1)-dwX, (((*pEnt)->dwvPos[1]+(*pEnt)->dwvPos[3])>>1)-dwY,
				(((*pEnt)->pvPaths[1]->dwvPos[0]+(*pEnt)->pvPaths[1]->dwvPos[2])>>1)-dwX, (((*pEnt)->pvPaths[1]->dwvPos[1]+(*pEnt)->pvPaths[1]->dwvPos[3])>>1)-dwY,
				8, makecol(255, dwR, dwR));
		}
	}
}

void DrawMapEnts(BITMAP* pBmp, map_entity_list_t& svEnts, int dwX, int dwY)
{
	for(map_entity_list_t::iterator pEnt=svEnts.begin(); pEnt!=svEnts.end(); ++pEnt)
	{
		if((*pEnt)->pType->pBmp)
		{
			draw_sprite(pBmp, (*pEnt)->pType->pBmp, (*pEnt)->dwvPos[0]+dwX+1, (*pEnt)->dwvPos[1]+dwY+1);
		}
		else if((*pEnt)->pType->dwColor)
		{
			drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
			rectfill(pBmp, (*pEnt)->dwvPos[0]+dwX, (*pEnt)->dwvPos[1]+dwY, (*pEnt)->dwvPos[2]+dwX, (*pEnt)->dwvPos[3]+dwY, (*pEnt)->pType->dwColor);
			solid_mode();
		}
		else
		{
			line(pBmp, (*pEnt)->dwvPos[0]+dwX, (*pEnt)->dwvPos[1]+dwY, (*pEnt)->dwvPos[2]+dwX, (*pEnt)->dwvPos[3]+dwY, (*pEnt)->pType->dwBorder);
			line(pBmp, (*pEnt)->dwvPos[0]+dwX, (*pEnt)->dwvPos[3]+dwY, (*pEnt)->dwvPos[2]+dwX, (*pEnt)->dwvPos[1]+dwY, (*pEnt)->pType->dwBorder);
		}
		rect(pBmp, (*pEnt)->dwvPos[0]+dwX, (*pEnt)->dwvPos[1]+dwY, (*pEnt)->dwvPos[2]+dwX, (*pEnt)->dwvPos[3]+dwY, (*pEnt)->pType->dwBorder);
		if((*pEnt)->pType->bShowName)
		{
			textout_centre(pBmp, font, (*pEnt)->sName.c_str(), (((*pEnt)->dwvPos[0]+(*pEnt)->dwvPos[2])>>1)+dwX, (((*pEnt)->dwvPos[1]+(*pEnt)->dwvPos[3])>>1)+dwY, makecol(0, 0, 0));
		}
	}
}

map_entity_p MapEntAtPoint(map_entity_list_t& svEnts, int dwX, int dwY)
{
	for(map_entity_list_t::reverse_iterator pEnt=svEnts.rbegin(); pEnt!=svEnts.rend(); ++pEnt)
	{
		if(dwX >= (*pEnt)->dwvPos[0]  &&  dwY >= (*pEnt)->dwvPos[1]  &&  dwX <= (*pEnt)->dwvPos[2]  &&  dwY <= (*pEnt)->dwvPos[3])
		{
			return *pEnt;
		}
	}

	return NULL;
}

void gb_fputs(string sBuff, PACKFILE* pFile)
{
	int dwSize = sBuff.size()+1;
	pack_fwrite(&dwSize, sizeof(int), pFile);
	pack_fwrite(sBuff.c_str(), dwSize, pFile);
}

void gb_fgets(string& sBuff, PACKFILE* pFile)
{
	char szBuff[256];
	int dwSize;
	pack_fread(&dwSize, sizeof(int), pFile);
	pack_fread(szBuff, dwSize, pFile);
	sBuff = szBuff;
}

void SaveMapEnts(map_entity_list_t& svEnts, string sDir)
{
	PACKFILE* pFile = pack_fopen((sDir+"Ents.me").c_str(), "w");
	if(!pFile)
		return;

	int dwVersion = 1;
	pack_fwrite(&dwVersion, sizeof(int), pFile);

	int dwCount = svEnts.size();
	pack_fwrite(&dwCount, sizeof(int), pFile);

	for(map_entity_list_t::iterator pEnt=svEnts.begin(); pEnt!=svEnts.end(); ++pEnt)
	{
		gb_fputs((*pEnt)->pType->szName, pFile);
		gb_fputs((*pEnt)->sName.c_str(), pFile);
		pack_fwrite(&(*pEnt)->dwvPos[0], sizeof(int), pFile);
		pack_fwrite(&(*pEnt)->dwvPos[1], sizeof(int), pFile);
		pack_fwrite(&(*pEnt)->dwvPos[2], sizeof(int), pFile);
		pack_fwrite(&(*pEnt)->dwvPos[3], sizeof(int), pFile);
		gb_fputs((*pEnt)->svVars[0].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[1].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[2].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[3].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[4].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[5].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[6].c_str(), pFile);
		gb_fputs((*pEnt)->svVars[7].c_str(), pFile);
	}

	pack_fclose(pFile);
}

map_entity_p FindMapEntityByName(map_entity_list_t& svEnts, string sName)
{
	for(map_entity_list_t::iterator pEnt=svEnts.begin(); pEnt!=svEnts.end(); ++pEnt)
	{
		if((*pEnt)->sName == sName)
			return *pEnt;
	}

	return NULL;
}

void ResolvePaths(map_entity_list_t& svEnts)
{
	for(map_entity_list_t::iterator pEnt=svEnts.begin(); pEnt!=svEnts.end(); ++pEnt)
	{
		if((*pEnt)->pType->svVars[0]  &&  ((*pEnt)->pType->svVars[0])[0] == '%'  &&  !(*pEnt)->svVars[0].empty())
			(*pEnt)->pvPaths[0] = FindMapEntityByName(svEnts, (*pEnt)->svVars[0]);
		if((*pEnt)->pType->svVars[1]  &&  ((*pEnt)->pType->svVars[1])[1] == '%'  &&  !(*pEnt)->svVars[1].empty())
			(*pEnt)->pvPaths[1] = FindMapEntityByName(svEnts, (*pEnt)->svVars[1]);
	}
}

void LoadMapEnts(map_ent_type_list_t& svEntTypes, map_entity_list_t& svEnts, string sDir)
{
	PACKFILE* pFile = pack_fopen((sDir+"Ents.me").c_str(), "r");
	if(!pFile)
		return;

	svEnts.clear();

	int dwVersion;
	pack_fread(&dwVersion, sizeof(int), pFile);

	int dwCount;
	pack_fread(&dwCount, sizeof(int), pFile);

	string sBuff;
	map_entity_p pEnt;
	for(int lp=0; lp<dwCount; lp++)
	{
		gb_fgets(sBuff, pFile);
		pEnt = NewMapEntity(svEnts, FindEntType(svEntTypes, sBuff), 0, 0);

		gb_fgets(pEnt->sName, pFile);

		pack_fread(&pEnt->dwvPos[0], sizeof(int), pFile);
		pack_fread(&pEnt->dwvPos[1], sizeof(int), pFile);
		pack_fread(&pEnt->dwvPos[2], sizeof(int), pFile);
		pack_fread(&pEnt->dwvPos[3], sizeof(int), pFile);

		gb_fgets(pEnt->svVars[0], pFile);
		gb_fgets(pEnt->svVars[1], pFile);
		gb_fgets(pEnt->svVars[2], pFile);
		gb_fgets(pEnt->svVars[3], pFile);
		gb_fgets(pEnt->svVars[4], pFile);
		gb_fgets(pEnt->svVars[5], pFile);
		gb_fgets(pEnt->svVars[6], pFile);
		gb_fgets(pEnt->svVars[7], pFile);

		pEnt->pvPaths[0] = NULL;
		pEnt->pvPaths[1] = NULL;
	}

	pack_fclose(pFile);

	ResolvePaths(svEnts);
}
