#ifndef DLL_SHARED_H
#define DLL_SHARED_H

#include <string>
#include <vector>
#include <map>
#include <list>

using namespace std;


#define RT_UP				0
#define RT_DOWN				1
#define RT_LEFT				2
#define RT_RIGHT			3
#define RT_WATER			4

#define MAKE_VER(major, minor, letter) (((major&0xFF)<<24)|((minor&0xFFFF)<<8)|(letter&0xFF))

struct gb_ltstr
{
  bool operator()(const string s1, const string s2) const
  {
    return s1 < s2;
  }
};


// +++++ Game typedefs +++++

typedef struct particle_s {
	bool					bActive;
	int						dwDeath;
	float					fvPos[2];
	float					fvVel[2];
	float					fAngle;
	bool					bDraw;
	bool					bLarge;
	bool					bWind;
	bool					bGravity;
	int						dwColor;
	BITMAP*					pBmp;
	void*					pData;

	void					(*m_Run)(particle_s* pPtc, int dwTime);
	void					(*m_Touch)(particle_s* pPtc, struct entity_s* pEnt, float* fvClip);
} particle_t, *particle_p;

typedef struct col_circle_s {
	float					fvOffset[2];
	float					fRadius;
} col_circle_t, *col_circle_p;
typedef vector<col_circle_t> col_circle_list_t;

typedef struct entity_s {
	string					sName;
	string					sClass;
	float					fvPos[2];
	float					fvVel[2];
	float					fvForce[2];
	float					fvSize[2];
	float					fvGravityMultiplier[2];
	float					fvGround[4];
	bool					bTakeDamage;
	bool					bGarbage;
	int						dwNextAnim;

	bool					bActive;
	bool					bVisible;
	bool					bAnimated;
	bool					bMobile;

//	col_circle_list_t		svColliders;

	BITMAP*					pSprite;
	void*					pvData[4];

	void					(*m_Draw)(entity_s* pEnt, BITMAP* pDest, int dwX, int dwY);
	void					(*m_Run)(entity_s* pEnt, int dwTime);
	void					(*m_Anim)(entity_s* pEnt, int dwTime);
	void					(*m_Touch)(entity_s* pEnt, entity_s* pOther);
	void					(*m_Trigger)(entity_s* pEnt, entity_s* pOther, string sV1, string sV2);
	void					(*m_Use)(entity_s* pEnt, entity_s* pPlayer);
	void					(*m_AltUse)(entity_s* pEnt, entity_s* pPlayer);
} entity_t, *entity_p;
typedef list<entity_p> entity_list_t;

typedef struct dll_exports_s {
	char*					szName;
	unsigned int			udwVersion;
} dll_exports_t, *dll_exports_p;

typedef struct trigger_zone_s {
	float					fvPos[4];
	bool					bActive;
	void					(*m_Trigger)(trigger_zone_s* pTrigger, entity_p pEntity, void* pData);
	void*					pData;
} trigger_zone_t, *trigger_zone_p;
typedef list<trigger_zone_t> trigger_zone_list_t;

struct platform_s;
typedef platform_s* platform_p;

typedef struct pole_info_s {
	float					fvStart[2];
	float					fvDir[2];
	float					fLength;
	entity_p				pOwner;
} pole_info_t, *pole_info_p;

typedef struct grab_info_s {
	float					fvStart[2];
	entity_p				pOwner;
} grab_info_t, *grab_info_p;


//  Import info

typedef entity_p (*spawn_func_p)(string sType, string sName, int dwvPosition[], string svVars[]);
typedef map<const string, spawn_func_p, gb_ltstr> spawn_func_list_t;

typedef void (*dll_key_func_p)(bool bUp, int dwKey);

typedef struct dll_imports_s {
	unsigned int			udwVersion;

	spawn_func_list_t		pvSpawnFuncs;
	entity_list_t			pvEnts;
	dll_key_func_p			pvKeyFuncs[256];
	entity_p				pPlayer;

	int						dwvScrollPos[2];
	int						dwvMousePos[2];
	bool					bvMouseButtons[3];
	int						dwGameTime;

	void					(*m_Line)(BITMAP* pDst, int dwX1, int dwY1, int dwX2, int dwY2, int dwColor);
	void					(*m_RectFill)(BITMAP* pDst, int dwX1, int dwY1, int dwX2, int dwY2, int dwColor);
	void					(*m_Rect)(BITMAP* pDst, int dwX1, int dwY1, int dwX2, int dwY2, int dwColor);

	BITMAP*					(*m_LoadSprite)(string sFile);
	void					(*m_DrawSprite)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY);
	void					(*m_DrawSpriteVFlip)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY);
	void					(*m_DrawSpriteHFlip)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY);
	void					(*m_DrawSpriteVHFlip)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY);
	void					(*m_RotateSprite)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY, float fAngle);
	void					(*m_RotateSpriteVFlip)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY, float fAngle);
	void					(*m_PivotSprite)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY, int dwCX, int dwCY, float fAngle);
	void					(*m_PivotSpriteVFlip)(BITMAP* pDst, BITMAP* pSrc, int dwX, int dwY, int dwCX, int dwCY, float fAngle);

	entity_p				(*m_NewEntity)();
	entity_p				(*m_FindEntityByName)(string sName, entity_p pStart);
	entity_p				(*m_FindEntityAtPoint)(float fX, float fY, entity_p pStart);
	entity_p				(*m_FindEntityInBox)(float fX1, float fY1, float fX2, float fY2, entity_p pStart);
	void					(*m_EntityVisible)(entity_p pEnt);
	void					(*m_EntityInvisible)(entity_p pEnt);
	void					(*m_EntityActive)(entity_p pEnt);
	void					(*m_EntityInactive)(entity_p pEnt);
	void					(*m_EntityAnimated)(entity_p pEnt);
	void					(*m_EntityInanimate)(entity_p pEnt);
	void					(*m_EntityMobile)(entity_p pEnt);
	void					(*m_EntityStatic)(entity_p pEnt);
	void					(*m_SetSpawnFunc)(string sName, spawn_func_p pFunc);

	float					(*m_TraceLine)(float fX1, float fY1, float& fX2, float& fY2, float* fvClip);
	bool					(*m_GetPoleAtPoint)(float fX, float fY, float fRange, pole_info_p pPole);
	bool					(*m_GetGrabAtPoint)(float fX, float fY, float fRange, bool bLeft, bool bRight, grab_info_p pPole);

	particle_p				(*m_CreateParticle)();
	void					(*m_DeleteParticle)(particle_p pPtc);

	void					(*m_LoadLevel)(string sDir);
	void					(*m_SetTile)(int dwX, int dwY, int dwLayer, int dwValue);
	int						(*m_GetTile)(int dwX, int dwY, int dwLayer);

	SAMPLE*					(*m_LoadSample)(string sName);
	void					(*m_PlaySample)(SAMPLE* pSample, float fVolume, float fPan, float fPitch, int dwPriority);
	void					(*m_PlaySampleAtPoint)(SAMPLE* pSample, float fX, float fY, float fVolume, float fPitch, int dwPriority);

	trigger_zone_p			(*m_AddTrigger)();

	platform_p				(*m_AddPlatform)(entity_p pEnt);
	void					(*m_PlatformAddBlocker)(platform_p pPlatform, int dwX1, int dwY1, int dwX2, int dwY2);
	void					(*m_PlatformAddPole)(platform_p pPlatform, int dwX1, int dwY1, int dwX2, int dwY2);
	void					(*m_PlatformAddLeftGrab)(platform_p pPlatform, int dwX1, int dwY1);
	void					(*m_PlatformAddRightGrab)(platform_p pPlatform, int dwX1, int dwY1);

	void					(*m_CreateReflection)(int dwType, int dwX1, int dwY1, int dwX2, int dwY2);
} dll_imports_t, *dll_imports_p;

// ----- Game typedefs -----




// +++++ Editor typedefs +++++

typedef struct map_ent_type_s {
	char*					szName;
	int						dwvSize[2];
	bool					bSizeable;
	bool					bShowName;
	int						dwColor;
	int						dwBorder;
	char*					svVars[8];
	char*					svVarDef[8];
	char*					szBmp;
	BITMAP*					pBmp;
} map_ent_type_t, *map_ent_type_p;

typedef struct map_entity_s {
	string					sName;
	map_ent_type_p			pType;
	int						dwvPos[4];
	string					svVars[8];
	map_entity_s*			pvPaths[2];
} map_entity_t, *map_entity_p;

typedef vector<map_entity_p> map_entity_list_t;
typedef vector<map_ent_type_p> map_ent_type_list_t;

typedef struct dll_edit_exports_s {
	char*					szName;

	map_ent_type_list_t		pMapEnts;
} dll_edit_exports_t, *dll_edit_exports_p;

// ----- Editor typedefs -----

extern dll_imports_p		g_pImports;
extern dll_exports_p		g_pExports;

#endif // DLL_SHARED_H
